
/*
 * RFID Akkuscan Version 1.10 by Sven Peitz http://www.rfidakkuscan.de 

 Volle Funktion aller einzelnen Komponenten
 Gerät ist mit 10 Akkus bestückt und wird im Feld getestet
 Erste Tests erfolgreich, nur ein Piezo buzzer müsste noch getestet werden
 Da das booten derzeit ca.3 Sekunden dauert muss noch der bootloader optimiert werden.
 Es wurde das Fuse Flag von D8 original auf D9 geändert.
 sudo avrdude -c usbasp -v -p m2560 -U hfuse:w:0xd9:m
 sudo avrdude -c usbasp -v -p m2560 -U hfuse:w:0xd8:m
 Datum und Uhrzeit kann per Serielle geändert werden
 Teststring:  befehl(string_1byte);akkutyp(string_5byte);Akkuname(string_16byte);Zellenzahl(int_2byte);ah(int_2byte);mah(int_2byte);
             maxladestrom_A(int_2byte);maxladestrom_mA(int_2byte);maxentladestrom_A(int_2byte);maxentladestrom_mA(int_2byte);
             tt_int_2byte;mm_int_2byte;yy_int_2byte;Anzalhl Ladungen (2 Byte); Euro (2Byte);Eurocent(2Byte)
 Teststring    n;Lipo ;SLS XTRON     ;12;0;850;3;2;25;70;31;01;13;4;5;99;
               n;Lipo ;SLS XTRON     ;12;0;850;3;2;25;70;31;01;13;4;5;99;
               n;Lipo ;TurnigyNanotec;12;3;850;10;50;235;70;31;12;13;10;5;99;
               
               Auslesen der akku.csv Datei auf SD Karte
               r;
               
               Löschen der akku.csv Datei auf SD Karte
               d;
               
               Setzen von Uhrzeit und Datum Format: t;year;month;day;hour;minute;secunde;
               t;2013;2;27;19;36;00;
               
               e;
               Diese Funktion ließt einen vorhandenen Chip neu ein erstellt eine neue ID und einen neuen Datensatz auf der Karte.
               Wird benötigt um nach einem löschen der akkus.csv die Daten von den Chips neu auf SD Karte einzulesen (2 mal scannen).
              
               v1.10 Dokumentationsfehler behoben Doku Kaufdatum war Jahr und Tag vertauscht "tt_int_2byte;mm_int_2byte;yy_int_2byte"
               
               v1.09 ab jetzt können auch NDEF vorformatierte Tag benutzt werden. Die Formatierung wird automatisch erkannt.
                * NEDF is based on code by www.electrodragon.com and modified/fixed by http://twitter.com/regnerischerTag and Sven
               
 */
 
#include <Adafruit_ST7735.h>
#include <Adafruit_GFX.h>
#include <Wire.h> 
#include <SD.h>
#include <SPI.h>
//#include <stdio.h>   // für freebyte
#include "RTClib.h"

#define	uchar	unsigned char
#define	uint	unsigned int

//Array of maximum length
#define MAX_LEN 16
//MF522 command word
#define PCD_IDLE              0x00               //NO action;Cancel the current command
#define PCD_AUTHENT           0x0E               //Authentication key
#define PCD_RECEIVE           0x08               //Receive Data
#define PCD_TRANSMIT          0x04               //send data
#define PCD_TRANSCEIVE        0x0C               //send and receive data
#define PCD_RESETPHASE        0x0F               //reset
#define PCD_CALCCRC           0x03               //CRC calculation

//Mifare_One card command word
#define PICC_REQIDL           0x26               //find antenna district not go to sleep
#define PICC_REQALL           0x52               //find the antenna region
#define PICC_ANTICOLL         0x93               //anti-collision
#define PICC_SElECTTAG        0x93               //Select card
#define PICC_ANTICOLL2        0x95		 // anticollision level 2
#define PICC_ANTICOLL3        0x97               // anticollision level 3
#define PICC_AUTHENT1A        0x60               //Verify A key
#define PICC_AUTHENT1B        0x61               //verify the B key
#define PICC_READ             0x30               //read block
#define PICC_WRITE            0xA0               //write block
#define PICC_DECREMENT        0xC0               //chargeback
#define PICC_INCREMENT        0xC1               //recharge
#define PICC_RESTORE          0xC2               //adjustable block of data to the buffer
#define PICC_TRANSFER         0xB0               //save the data in the buffer
#define PICC_HALT             0x50               //sleep

//And MF522 communicate returned error code
#define MI_OK                 0
#define MI_NOTAGERR           1
#define MI_ERR                2

//------------------MFRC522 Register---------------
//Page 0:Command and Status
#define     Reserved00            0x00    
#define     CommandReg            0x01    
#define     CommIEnReg            0x02    
#define     DivlEnReg             0x03    
#define     CommIrqReg            0x04    
#define     DivIrqReg             0x05
#define     ErrorReg              0x06    
#define     Status1Reg            0x07    
#define     Status2Reg            0x08    
#define     FIFODataReg           0x09
#define     FIFOLevelReg          0x0A
#define     WaterLevelReg         0x0B
#define     ControlReg            0x0C
#define     BitFramingReg         0x0D
#define     CollReg               0x0E
#define     Reserved01            0x0F
//Page 1:Command     
#define     Reserved10            0x10
#define     ModeReg               0x11
#define     TxModeReg             0x12
#define     RxModeReg             0x13
#define     TxControlReg          0x14
#define     TxAutoReg             0x15
#define     TxSelReg              0x16
#define     RxSelReg              0x17
#define     RxThresholdReg        0x18
#define     DemodReg              0x19
#define     Reserved11            0x1A
#define     Reserved12            0x1B
#define     MifareReg             0x1C
#define     Reserved13            0x1D
#define     Reserved14            0x1E
#define     SerialSpeedReg        0x1F
//Page 2:CFG    
#define     Reserved20            0x20  
#define     CRCResultRegM         0x21
#define     CRCResultRegL         0x22
#define     Reserved21            0x23
#define     ModWidthReg           0x24
#define     Reserved22            0x25
#define     RFCfgReg              0x26
#define     GsNReg                0x27
#define     CWGsPReg	          0x28
#define     ModGsPReg             0x29
#define     TModeReg              0x2A
#define     TPrescalerReg         0x2B
#define     TReloadRegH           0x2C
#define     TReloadRegL           0x2D
#define     TCounterValueRegH     0x2E
#define     TCounterValueRegL     0x2F
//Page 3:TestRegister     
#define     Reserved30            0x30
#define     TestSel1Reg           0x31
#define     TestSel2Reg           0x32
#define     TestPinEnReg          0x33
#define     TestPinValueReg       0x34
#define     TestBusReg            0x35
#define     AutoTestReg           0x36
#define     VersionReg            0x37
#define     AnalogTestReg         0x38
#define     TestDAC1Reg           0x39  
#define     TestDAC2Reg           0x3A   
#define     TestADCReg            0x3B   
#define     Reserved31            0x3C   
#define     Reserved32            0x3D   
#define     Reserved33            0x3E   
#define     Reserved34		  0x3F

//4 bytes Serial number of card, the 5th byte is crc
uchar serNum[5];
//7 bytes Serial number of card, the 8th byte is crc
uchar serNum7[8];
//buffer
//uchar str[MAX_LEN];

uchar defaultKeyA[16] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
uchar madKeyA[16] =     { 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5 };
uchar NDEFKeyA[16] =    { 0xD3, 0xF7, 0xD3, 0xF7, 0xD3, 0xF7 };

uchar  writeDataAkkuName[16]={ 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //Initialize 0 to RFID Register
uchar  writeDataAkkuDetails1[16]={ 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //Initialize 0 to RFID Register
uchar  writeDataAkkuDetails2[16]={ 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //Initialize 0 to RFID Register
uchar  writeDataAkkuDetails3[16]={ 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //Initialize 0 to RFID Register
uchar  writeDataAkkuDetails4[16]={ 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //Initialize 0 to RFID Register
uchar  writeDataAkkuDetails5[16]={ 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //Initialize 0 to RFID Register
uchar  writeDataAkkuDetails6[16]={ 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //Initialize 0 to RFID Register
//uchar  writeDataAkkuDetails7[16]={ 
//  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //Initialize 0 to RFID Register
//

/*

 The circuit:
 * analog sensors on analog ins 0, 1, and 2
 * SD card attached to SPI bus as follows:
 ** MOSI - pin 11   // Pin 15 am AVR328
 ** MISO - pin 12   // Pin 16 am AVR328
 ** CLK - pin 13    // Pin 17 am AVR328
 ** CS - pin 4  also 10    // Pin 14 am AVR328
 */
// On the Ethernet Shield, CS is pin 4. Note that even if it's not
// used as the CS pin, the hardware CS pin (10 on most Arduino boards,
// 53 on the Mega) must be left as an output or the SD library
// functions will not work.

//define für Display und SD
//#define tft_sclk 52
//#define tft_mosi 51
#define tft_cs   47
#define tft_dc   49
#define tft_rst  48


// Option 1: use any pins but a little slower, but only in use with normal Pins, standard mosi,miso,sclk wont work 
// try Option 2
//ST7735 tft = ST7735(tft_cs, tft_dc, tft_mosi, tft_sclk, tft_rst);  
Adafruit_ST7735 tft = Adafruit_ST7735(tft_cs, tft_dc, tft_rst);
//ST7735 tft = ST7735(tft_cs, tft_dc, tft_rst);
// For Arduino Uno/Duemilanove, etc
//  connect the SD card with MOSI going to pin 11, MISO going to pin 12 and SCK going to pin 13 (standard)
//  Then pin 4 goes to CS (or whatever you have set up)
#define SD_CS 53    // Set the chip select line to whatever you use (4 doesnt conflict with the library)

RTC_DS1307 RTC;
int addr = 0;
int akkuidInt = 0;
int usb = 0;   // merker für einfachen durchlauf
int akkuSensorPin = A0;    // select the input pin for voltage measuring
int akkuSensorValue = 0;  // variable to store the value coming from analog pin
float akkuSpannung;       // variable to store voltage 
File dataFile;            // varibale to set up SD Card File
String inputString = "";  // variable for input String from Serial
boolean stringComplete = false;  // whether the string is complete
const int rfidWritePin = 22;     // the number of the pushbutton pin INT 0 
int rfidWritePinState = 0;     // variable for reading the pushbutton status
const int piezoBuzzer = 43;    //the number for the piezo buzzer
boolean ndef=false;

//some vars for the card data arrays
String akkuscantag = "";   
String akkuscanmonat = "";
String akkuscanjahr = "";
String zellenzahl="";
String akkuah="";
String akkumah="";
String akkukauftag="";
String akkukaufmonat="";
String akkukaufjahr="";
String akkuscanstunde="";
String akkuscanminute="";
String akkuscansekunde="";
String akkutyp="";
String akkuid="";
String ladungen="";
String maxentladestromA="";
String maxentladestrommA="";
String maxladestromA="";
String maxladestrommA="";
String akkuname="";
String reserve="";
String preiseuro="";
String preiseurocent="";
//String lastgeladenemah="";
String lastgeladenema="";
String maxgeladenemah="";
String maxgeladenema="";

/////////////////////////////////////////////////////////////////////
//set the pin for RFID
/////////////////////////////////////////////////////////////////////
const int chipSelectPin = 41;    //RFID 
const int NRSTPD = 40;           //NSRST_CS 

////with next statc int ... C stuff you can look for free sram bytes
//static int freeRam () {
//  extern int __heap_start, *__brkval; 
//  int v; 
//  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
//}
//
//unsigned long time;

String splitString(String s, char parser,int index);  // for splitstring void

void setup(){
  Serial.begin(9600);            // set Serial to 9600 baud
  // pinMode(10, OUTPUT);
  pinMode(chipSelectPin,OUTPUT);            // Set digital pin OUTPUT to connect it to the RFID /ENABLE pin chip select
  digitalWrite(chipSelectPin, LOW);         // Activate the RFID reader
  pinMode(NRSTPD,OUTPUT);                   // Set digital Not Reset and Power-down RFID
  digitalWrite(NRSTPD, HIGH);               // Set digital Not Reset and Power-down RFID
  pinMode(tft_cs, OUTPUT);                  //set digital pin for TFT chip select
  digitalWrite(tft_cs, HIGH);               //set digital pin for TFT chip select
  pinMode(piezoBuzzer, OUTPUT);                  //set digital pin for piezoBuzzer
  digitalWrite(piezoBuzzer, LOW);                //set digital pin for piezoBuzzer LOW
  // initialize the pushbutton pin as an input:
  pinMode(rfidWritePin, INPUT);
  //rtc
  Wire.begin();
  RTC.begin();

  // if (! RTC.isrunning()) {
  //  Serial.println("RTC is NOT running!");
  // following line sets the RTC to the date & time this sketch was compiled
  //RTC.adjust(DateTime(__DATE__, __TIME__));
  //  RTC.adjust(DateTime(2013,1,29,21,10,00));
  // }
  //rtc end
  tft.initR(INITR_REDTAB);               // initialize a ST7735R chip 1.8"TFT Diplay
  //  tft.setTextWrap(false);            // Allow text to run off right edge
  tft.fillScreen(ST7735_BLACK);          // init complete screen with black color
  MFRC522_Init();                        // RFDI Chip INIT
  inputString.reserve(90);               // reserve 80 bytes for the inputString: Serial
  tft.setRotation(tft.getRotation()+1);  // rotate TFT Display 90 degrees
  tft.setCursor(0, 0);                   // set cursor to Position 0,0   colum, row
  tft.setTextColor(ST7735_GREEN,ST7735_BLACK);  // set text color to green and background to black 
  // background is optional you need it to overwrite existing chars
  tft.setTextSize(1);                           // set Text size to standard 1, for bigger oner grow up the number 1,2,3,4 etc.
  if (!SD.begin(SD_CS)){                      // see if the card is present and can be initialized:
    tft.setCursor(0, 0);
    tft.println("Card failed");
    while (1);                                  // don't do anything more:
    return;
  }
    
  // dataFile = SD.open("akkus.csv, FILE_READ");              //look for akkus.csv file on SD card
  //  if (!dataFile) {                             // if there is no existig file..
  //        dataFile = SD.open("akkus.csv", FILE_WRITE);//create the file
  //        dataFile.close();                           //create the file
  //        tft.setCursor(0, 0);
  //        tft.println("didnt find akkus.csv");
  //        tft.println("at startup. create it");
  //        while(1);
  //        //     Serial.println("didnt find akkus.csv");
  //  }
//  DateTime now = RTC.now();
//  akkuscantag = String(now.day());  // aus RTC
//  akkuscanmonat = String(now.month()); // aus RTC
//  int jahr = now.year(); // aus RTC
//  akkuscanjahr = String(jahr-2000);
//  Serial.print(akkuscantag+akkuscanmonat+akkuscanjahr);
//  Serial.println("init done");
}


void loop(){
  // Akku Spannung berechnen 13V Eingang (16K --- 10K) macht bei 13V/1024 = 0.0126953125
  // 2 S Akku sind voll bei 8,4V = AD657; leer bei 7,2V = AD567
  // 3 S Akku sind voll bei 12,6V = AD985; leer bei 10,8V = AD849
  //float akkuSpannung = akkuSensorValue*0.0126953125;
  //float akkuSpannung = akkuSensorValue*0.012792969;
  //  akkuSensorValue = analogRead(akkuSensorPin);    // read the value from the sensor:
  //  akkuSpannung = akkuSensorValue*0.012792969;     //calc akkuvoltage with faktor to get normal Voltage example 11.85V (float var) 
  akkuSensorValue = analogRead(akkuSensorPin);    // read the value from the sensor:
  akkuSpannung = akkuSensorValue*0.012792969;     //calc akkuvoltage with faktor to get normal Voltage example 11.85V (float var) 
  rfidWritePinState = digitalRead(rfidWritePin);
  // check if the pushbutton is pressed.
  // if it is, the buttonState is HIGH:
  if (rfidWritePinState == HIGH) {     
    //do something read
    if (akkuSensorValue > 200){
      displayStatus(1);
      
      rfidDataConvert(1);   // rfid Daten chip auslesen Daten auf dem TFT anzeigen
    }
    else if (akkuSensorValue == 0){
      tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
      tft.setTextSize(2);
      tft.setCursor(0, 0);
      tft.println("   PC Modus  ");
      tft.setTextSize(1);
      tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
      if (stringComplete){ 
        serialData();  // serielle Daten abfragen ggf neuen Akkutyp anlegen und auf rfid / SD Speicher  schreiben
      }

    }

  } 
  else {
    // do something write
    if (akkuSensorValue > 500){
       displayStatus(2);
       rfidDataConvert(2);   // rfid Daten chip auslesen und mit aktuellem scan datum versehen akkuid um eins erhöhen und schreiben
    }
      else if (akkuSensorValue == 0){
      tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
      tft.setTextSize(2);
      tft.setCursor(0, 0);
      tft.println("   PC Modus  ");
      tft.setTextSize(1);
      tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
      if (stringComplete){ 
        serialData();  // serielle Daten abfragen ggf neuen Akkutyp anlegen und auf rfid / SD Speicher  schreiben
      }
    }
}
  
  //    Serial.print("Free Ram (Byte) : ");
  //    Serial.println(freeRam());
  //  tft.setCursor(0, 10);
  //  tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
  //  tft.println("Free Ram (Byte) : ");  
  //  time=millis()-time;
  //  tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
  //  tft.println(freeRam());
}

///////////////////////////////////////////////////////////////// Hier  ENDE LOOP ////////////////////////////////////////////////////////////

void displayStatus(int i){
  akkuSensorValue = analogRead(akkuSensorPin);    // read the value from the sensor:
  akkuSpannung = akkuSensorValue*0.012792969;     //calc akkuvoltage with faktor to get normal Voltage example 11.85V (float var) 
  String softwareVersion="v1.09";
    if (i==1){
      tft.setTextColor(ST7735_GREEN,ST7735_BLACK);        
      tft.setTextSize(2);
      tft.setCursor(0, 0);
      tft.println("  Akku Info ");
      tft.setTextSize(1);
      tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
      if (akkuSensorValue >= 700){
        tft.setCursor(10, 20);
        tft.print("3S Akku: ");  
        tft.print(akkuSpannung, 2);
        tft.print("V ");
        if (((akkuSensorValue-826)*100)/140 <= 25.00){
          tft.setTextColor(ST7735_RED,ST7735_BLACK); 
        }
        tft.print(((akkuSensorValue-826)*100)/140);
        tft.print("%");
        tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
        tft.println(softwareVersion);
      }  
      else{
        tft.setCursor(10, 20);
        tft.print("2S Akku: ");  
        tft.print(akkuSpannung, 2);
        tft.print("V ");
        if (((akkuSensorValue-561)*100)/90 <= 25.00){
          tft.setTextColor(ST7735_RED,ST7735_BLACK); 
        }
        tft.print(((akkuSensorValue-561)*100)/90);
        tft.print("% ");
         tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
        tft.println(softwareVersion);
      }
     }
     if (i==2){
      tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
      tft.setTextSize(2);
      tft.setCursor(0, 0);
      tft.println("  Akku laden ");  
      tft.setTextSize(1);
      tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
      if (akkuSensorValue >= 700){
        tft.setCursor(10, 20);
        tft.print("3S Akku: ");  
        tft.print(akkuSpannung, 2);
        tft.print("V ");
        if (((akkuSensorValue-826)*100)/140 <= 25.00){
          tft.setTextColor(ST7735_RED,ST7735_BLACK); 
        }
        tft.print(((akkuSensorValue-826)*100)/140);
        tft.print("% ");
        tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
        tft.println(softwareVersion);
      }  
      else{
        tft.setCursor(10, 20);
        tft.print("2S Akku: ");  
        tft.print(akkuSpannung, 2);
        tft.print("V ");
        if (((akkuSensorValue-561)*100)/90 <= 25.00){
          tft.setTextColor(ST7735_RED,ST7735_BLACK); 
        }
        tft.print(((akkuSensorValue-561)*100)/90);
        tft.print("% ");
         tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
        tft.println(softwareVersion);
      }
}
}

void rfidDataConvert(int i){
  int schalterwechsel = 0;
  //  Debug Infos
  //  Serial.print(akkutyp + zellenzahl + "S " + akkuah + "." + akkumah + "Ah,Ladestrom: " + maxladestromA + "." + maxladestrommA);
  //  Serial.print("A,Entladestrom: " + maxentladestromA + "." + maxentladestrommA + "A,Kaufdatum: " + akkukauftag + "." + akkukaufmonat + ".20" + akkukaufjahr);
  //  Serial.println(",Last Scan: " + akkuscantag + "." + akkuscanmonat + "." + "20" + akkuscanjahr + ",Akuuid: " + akkuid + ",Ladungen : " + ladungen);
  //  String displayNachricht = String(array + akkutyp + zellenzahl + "S " + akkuah + "." + akkumah + "Ah,Ladestrom: " + maxladestromA + "." + maxladestrommA + "A,Entladestrom: " + maxentladestromA + "." + maxentladestrommA + "A,Kaufdatum: " + akkukauftag + "." + akkukaufmonat + ".20" + akkukaufjahr + ",Last Scan: " + akkuscantag + "." + akkuscanmonat + "." + "20" + akkuscanjahr + ",Akuuid: " + akkuid);
  if (i == 1){   
      while((rfid(1)) !=1){
           rfidWritePinState = digitalRead(rfidWritePin);
          // check if the pushbutton is pressed.
          // if it is, the buttonState is HIGH:
          if (rfidWritePinState == LOW) {     
              schalterwechsel = 1;
              MFRC522_Init();                        // RFDI Chip INIT
              tft.fillScreen(ST7735_BLACK);          // init complete screen with black color
              break;
          }
      }
      if (schalterwechsel == 0){
          digitalWrite(piezoBuzzer, HIGH);               //set digital pin for piezoBuzzer
          delay(10);
          digitalWrite(piezoBuzzer, LOW);               //set digital pin for piezoBuzzer
         tft.fillScreen(ST7735_BLACK);
         displayStatus(1);
         akkuDetailsAnzeigen();
      }
    }
  if (i == 2){   
    // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und array 6 und SD Karte schreiben
    // jetzt schreibe auf ARRAY3 auf RFID und SD Karte Funktionsaufruf RFID Array 3 schreiben
    while((rfid(2)) !=2){   // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und array 6 und SD Karte schreiben
          rfidWritePinState = digitalRead(rfidWritePin);
          // check if the pushbutton is pressed.
          // if it is, the buttonState is HIGH:
          if (rfidWritePinState == HIGH) {     
              schalterwechsel = 1;
              MFRC522_Init();                        // RFDI Chip INIT
              tft.fillScreen(ST7735_BLACK);          // init complete screen with black color
              break;
          }
    }
    if (schalterwechsel == 0){
    digitalWrite(piezoBuzzer, HIGH);               //set digital pin for piezoBuzzer
    delay(10);
    digitalWrite(piezoBuzzer, LOW);               //set digital pin for piezoBuzzer
    dataFile = SD.open("akkus.csv", FILE_WRITE);
    // if the file opened okay, write to it:  writeDataAkkuDetails3
    if (dataFile){
      //  unsigned long pos = fileSize/114;
      int value1 = akkuid.toInt();
      //value ++;
      dataFile.seek(0);
      dataFile.seek((value1 *114)-114);  //Die richtige Position im SD Speicher finden eine Zeile hat 114 Byte inkl \n am ende
      //ausgelesene akkuid mal 114Byte findet das Ende der richtigen Zeile dann 114 Byte zurueck
      //Das ist die erste Stelle dort wird der Wert des Array geschrieben
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuName[x]);
      //  Serial.print(writeDataAkkuName[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails1[x]);
     //   Serial.print(writeDataAkkuDetails1[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails2[x]);
      //   Serial.print(writeDataAkkuDetails2[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails3[x]);
      //   Serial.print(writeDataAkkuDetails3[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails4[x]);
      //   Serial.print(writeDataAkkuDetails4[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails5[x]);
      //   Serial.print(writeDataAkkuDetails5[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails6[x]);
       //  Serial.print(writeDataAkkuDetails6[x]);
      }
//      for (int x=0; x < 4 ; x++){
//        //Serial.print(dataArray[x]);
//        dataFile.write(serNum[x]);
//      }
      dataFile.println();       // 2 byte am ende für \n
      // close the file: to write the Data
      dataFile.close();
    }
    else{
      // if the file didn't open, print an error:
      tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
      tft.println("error opening akkus.csv ");
      tft.println("for read while change");
      tft.println("a existing akku");
      //   Serial.println("error opening akkus.csv for read");
      while(1);
    }
    tft.fillScreen(ST7735_BLACK);
    displayStatus(2);
    akkuDetailsAnzeigen();  
   }		   
 }
 
  
}  // End of rfidDataConvert

void serialData(){
  if (stringComplete){
    String befehl=splitString(inputString,';',0);
    akkutyp=splitString(inputString,';',1);
    akkuname=splitString(inputString,';',2);
    zellenzahl=splitString(inputString,';',3);
    akkuah=splitString(inputString,';',4);
    akkumah=splitString(inputString,';',5);
    maxladestromA=splitString(inputString,';',6);
    maxladestrommA=splitString(inputString,';',7);
    maxentladestromA=splitString(inputString,';',8);
    maxentladestrommA=splitString(inputString,';',9);
    akkukauftag=splitString(inputString,';',10);
    akkukaufmonat=splitString(inputString,';',11);
    akkukaufjahr=splitString(inputString,';',12);
    ladungen=splitString(inputString,';',13);
    preiseuro=splitString(inputString,';',14);
    preiseurocent=splitString(inputString,';',15);
    lastgeladenema="0";
    maxgeladenemah="0";
    maxgeladenema="000";
    DateTime now = RTC.now();
    akkuscantag = String(now.day());  // aus RTC
    akkuscanmonat = String(now.month()); // aus RTC
    int jahr = now.year(); // aus RTC
    akkuscanjahr = String(jahr-2000);
    akkuscanstunde = String(now.hour());  // aus RTC
    akkuscanminute = String(now.minute());  // aus RTC
    akkuscansekunde = String(now.second());  // aus RTC
    //  Serial.println("array ist zerlegt" + akkuscantag + akkuscanmonat +akkuscanjahr );
    //  Serial.println(inputString); 
    //                      if ((sizeof(inputString) !=82)){
    //                          Serial.flush();
    //                          inputString="";
    //                          stringComplete=false;
    //                      }
    if (befehl== "d"){
      SD.remove("akkus.csv");       //Datei löschen
      dataFile = SD.open("akkus.csv", FILE_WRITE);
      dataFile.close();
      inputString = "";
      stringComplete = false;
      return;
    }
    if (befehl== "e"){
      tft.fillScreen(ST7735_BLACK);   
      tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
      tft.setTextSize(2);
      tft.setCursor(0, 0);
      tft.println("  Chip lesen ");
      tft.println();
      tft.setTextSize(1);
      int value1=0;
     MFRC522_Init();                        // RFDI Chip INIT
     
      // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und array 6 und SD Karte schreiben
    // jetzt schreibe auf ARRAY3 auf RFID und SD Karte Funktionsaufruf RFID Array 3 schreiben
       while((rfid(1)) !=1){   // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und array 6 und SD Karte schreiben
      }
      digitalWrite(piezoBuzzer, HIGH);               //set digital pin for piezoBuzzer
      delay(10);
      digitalWrite(piezoBuzzer, LOW);               //set digital pin for piezoBuzzer
     
      
      dataFile = SD.open("akkus.csv", FILE_READ);
      // if the file opened okay, read from it:

      if (dataFile){
        // read from the file until there's nothing else in it:
        // Serial.println("if datafile");
        unsigned long fileSize = dataFile.size();

        dataFile.seek((fileSize-114)+48);  //vorher 25  Setzte position Dateiende -114 +48 anfang 4 16byte array letzter Datensatz 
        //int y=0;
        String msg = "";
        while (dataFile.available()){
          msg += char(dataFile.read()); 
          //   Serial.print(msg);
        }
        //  Serial.write(dataFile.read());
        dataFile.close();
        String id = "";
        id = splitString(msg,';',3);    /// akkuid steht an elfter stelle
        //       String lad=splitString(msg,';',1);
        // Serial.println(id);
        int value = id.toInt();
        // Serial.println(msg);
        //       int ladungen = lad.toInt();
        akkuidInt = value +1;
        akkuid = String(akkuidInt);
        //    Serial.println("akkuid : "+ akkuid);   //Debug Information
        //   Serial.println(msg);  //Debug Information
        // close the file: to write the Data
        //dataFile.close();
      }
      else{
        // if the file didn't open, print an error:
        tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
        tft.setCursor(10, 30);
        tft.println("error opening akkus.csv"); 
        tft.println("for read/write an new akku");
        while(1);
      }
  
    char array0[16];
    int i = 0;
    for  (i=0; i<16; i++){
          array0[i] = writeDataAkkuDetails3[i];
    }
    akkuscantag = splitString(array0,';',0);    /// akkuid steht an elfter stelle
    akkuscanmonat = splitString(array0,';',1);    /// akkuid steht an elfter stelle
    akkuscanjahr = splitString(array0,';',2);    /// akkuid steht an elfter stelle
    ladungen = splitString(array0,';',4);    /// akkuid steht an elfter stelle
    value1 = akkuid.toInt();
    String val3 = (akkuscantag + ";" + akkuscanmonat + ";" + akkuscanjahr + ";" + akkuid  + ";" + ladungen + ";"); // 16 Bytearray
    
      //      Serial.println(val3.length());
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails3[x] = (val3[x]);
//        char mychar=(val3[x]);
//        dataArray[z] = mychar;
        // Serial.print(dataArray[z]);
 //       z++;
      }
    tft.setTextSize(2);
    tft.println("Nochmal lesen");
    MFRC522_Init();
    tft.setTextSize(1);
    while((rfid(0)) != 0){
          //      Serial.println(  rfid(0));  // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und SD Karte schreiben 
   //       Serial.println("jetzt neuen chip anlegen");
          //rfid(0);
        }
    digitalWrite(piezoBuzzer, HIGH);               //set digital pin for piezoBuzzer
    delay(10);
    digitalWrite(piezoBuzzer, LOW); 
    tft.fillScreen(ST7735_BLACK);   
                //set digital pin for piezoBuzzer
    dataFile = SD.open("akkus.csv", FILE_WRITE);
    // if the file opened okay, write to it:  writeDataAkkuDetails3
    if (dataFile){
      //  unsigned long pos = fileSize/114;
    //  int value1 = akkuid.toInt();
      //value ++;
      dataFile.seek(0);
      dataFile.seek((value1 *114)-114);  //Die richtige Position im SD Speicher finden eine Zeile hat 114 Byte inkl \n am ende
      //ausgelesene akkuid mal 114Byte findet das Ende der richtigen Zeile dann 114 Byte zurueck
      //Das ist die erste Stelle dort wird der Wert des Array geschrieben
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuName[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails1[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails2[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails3[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails4[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails5[x]);
      }
      for (int x=0; x <= 15 ; x++){
        //Serial.print(dataArray[x]);
        dataFile.write(writeDataAkkuDetails6[x]);
      }
//      for (int x=0; x < 4 ; x++){
//        //Serial.print(dataArray[x]);
//        dataFile.write(serNum[x]);
//      }
      dataFile.println();       // 2 byte am ende für \n
      // close the file: to write the Data
      dataFile.close();
    }
    else{
      // if the file didn't open, print an error:
      tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
      tft.println("error opening akkus.csv ");
      tft.println("for read while change");
      tft.println("a existing akku");
      //   Serial.println("error opening akkus.csv for read");
      while(1);
    }
    akkuDetailsAnzeigen();
    }
    
    if (befehl == "t"){
//      // Set Time and Date The String must be:  t;year;month;day;hour;minute;secunde;
       int jahr = akkutyp.toInt();
       int monat = akkuname.toInt();
       int tag = zellenzahl.toInt();
       int stunde = akkuah.toInt();
       int minuten = akkumah.toInt();
       int sekunden = maxladestromA.toInt();
//       RTC.adjust(DateTime(2013,1,29,21,10,00)); 
       RTC.adjust(DateTime(jahr,monat,tag,stunde,minuten,sekunden)); 
       DateTime now = RTC.now();
       if (! RTC.isrunning()) {
              Serial.println("RTC is NOT running!");
        }
        else{
            Serial.print("RTC DateTime: ");
            Serial.print(now.year(), DEC);
            Serial.print('/');
            Serial.print(now.month(), DEC);
            Serial.print('/');
            Serial.print(now.day(), DEC);
            Serial.print(' ');
            Serial.print(now.hour(), DEC);
            Serial.print(':');
            Serial.print(now.minute(), DEC);
            Serial.print(':');
            Serial.print(now.second(), DEC);
            Serial.println();
       }
    }
    
    if (befehl== "n"){
      // neuen Chip lesen um die Seriennnummer abzugreifen
      //    rfid(1);  // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und SD Karte schreiben
      // read akkuid from storage then inkremet 1
      // open the file. note that only one file can be open at a time,
      // so you have to close this one before opening another.
      tft.fillScreen(ST7735_BLACK);          // init complete screen with black color
      MFRC522_Init();                        // RFDI Chip INIT
      dataFile = SD.open("akkus.csv", FILE_READ);
      // if the file opened okay, read from it:

      if (dataFile){
        // read from the file until there's nothing else in it:
        // Serial.println("if datafile");
        unsigned long fileSize = dataFile.size();

        dataFile.seek((fileSize-114)+48);  //vorher 25  Setzte position Dateiende -114 +48 anfang 4 16byte array letzter Datensatz 
        //int y=0;
        String msg = "";
        while (dataFile.available()){
          msg += char(dataFile.read()); 
          //   Serial.print(msg);
        }
        //  Serial.write(dataFile.read());
        dataFile.close();
        String id = "";
        id = splitString(msg,';',3);    /// akkuid steht an elfter stelle
        //       String lad=splitString(msg,';',1);
        // Serial.println(id);
        int value = id.toInt();
        // Serial.println(msg);
        //       int ladungen = lad.toInt();
        akkuidInt = value +1;
        akkuid = String(akkuidInt);
        //    Serial.println("akkuid : "+ akkuid);   //Debug Information
        //   Serial.println(msg);  //Debug Information
        // close the file: to write the Data
        //dataFile.close();
      }
      else{
        // if the file didn't open, print an error:
        tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
        tft.setCursor(10, 30);
        tft.println("error opening akkus.csv"); 
        tft.println("for read/write an new akku");
        while(1);
      }
      //Akkuname ins 16Byte Array writeDataAkkuName packen zur Vorbereitung RFID Write 
      char dataArray[120];
      dataArray[0]=0;
      int z=0;
      String val0 =(akkuname + ";");
      //     Serial.println(val0.length());
      //writeDataAkkuName.toCharArray(val0, 15)  geht nicht weil uchar und char
      for (int x=0; x <=15  ; x++){
        writeDataAkkuName[x] = val0[x];
        char mychar=val0[x];
        dataArray[z] = mychar;
        //  Serial.print(dataArray[z]);
        z++;
      }
      //     Serial.println(dataArray);
      //    Serial.println(z);
      // Akkudetails1 ins 16Byte Array writeDataAkkuDetails1 packen zur Vorbereitung RFID Write
      String val1 = (zellenzahl + ";" + akkuah + ";" + akkumah + ";"); // 16 Bytearray
      //      Serial.println(val1.length());
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails1[x] = (val1[x]);
        char mychar=(val1[x]);
        // char mychar=writeDataAkkuDetails1[x];
        dataArray[z] = mychar;
        //	 Serial.print(dataArray[z]);
        z++;
      }
      //     Serial.println(dataArray);
      //        Serial.println(z);
      //    

      // Akkudetails1 (akkuid Kaufdatum und letzter scan) ins 16Byte Array writeDataAkkuDetails2 packen zur Vorbereitung RFID Write
      String val2 = (akkukauftag + ";" + akkukaufmonat + ";" + akkukaufjahr + ";"+ akkutyp + ";"); // 16 Bytearray
      //      Serial.println(val2.length());
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails2[x] = (val2[x]);
        char mychar=(val2[x]);
        dataArray[z] = mychar;
        // Serial.print(dataArray[z]);
        z++;
      }
      //Serial.println(dataArray);
      //   Serial.println(z);
      String val3 = (akkuscantag + ";" + akkuscanmonat + ";" + akkuscanjahr + ";" + akkuid  + ";" + ladungen + ";"); // 16 Bytearray
      //      Serial.println(val3.length());
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails3[x] = (val3[x]);
        char mychar=(val3[x]);
        dataArray[z] = mychar;
        // Serial.print(dataArray[z]);
        z++;
      }
      //Serial.println(dataArray);
      //        Serial.println(z);
      String val4 = (maxentladestromA + ";" + maxentladestrommA + ";" + maxladestromA + ";" + maxladestrommA + ";"); // 16 Bytearray
      //     Serial.println(val4.length());
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails4[x] = (val4[x]);
        char mychar=(val4[x]);
        dataArray[z] = mychar;
        //        Serial.print(dataArray[z]);
        z++;
      }

      //        Serial.println(z);
      String val5 = (preiseuro + ";" + preiseurocent + ";" + maxgeladenemah + ";" + maxgeladenema + ";"); // 16 Bytearray
      //     Serial.println(val4.length());
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails5[x] = (val5[x]);
        char mychar=(val5[x]);
        dataArray[z] = mychar;
        //        Serial.print(dataArray[z]);
        z++;
      }
      //        Serial.println(z);
      String val6 = (akkuscanstunde + ";" + akkuscanminute + ";" + akkuscansekunde + ";" + lastgeladenema + ";"); // 16 Bytearray
      //     Serial.println(val4.length());
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails6[x] = (val6[x]);
        char mychar=(val6[x]);
        dataArray[z] = mychar;
        //        Serial.print(dataArray[z]);
        z++;
      }
      //        Serial.println(z);
      //  String val7 = (serNum + ";"); // 16 Bytearray
      //     Serial.println(val4.length());
//      for (int x=0; x < 4  ; x++){
//        writeDataAkkuDetails7[x] = (serNum[x]);
//        char mychar=(serNum[x]);
//        dataArray[z] = mychar;
//        //        Serial.print(dataArray[z]);
//        z++;
//      }
      //  Speichern auf SD Karte
      // 80 byte
      // open the file. note that only one file can be open at a time,
      // so you have to close this one before opening another.
      //Serial.println("Now open for write");
      dataFile = SD.open("akkus.csv", FILE_WRITE);

      if (dataFile){
        // if the file is available, write to it:
        for (int x=0; x <112 ; x++){    // Achtung kleiner 112 damit nicht 113 zeichen geschrieben werden weil bei 0 angefangen wird.
          // Serial.print(dataArray[x]);
          // dataFile.write(dataArray[x]);
          dataFile.print(dataArray[x]);
          // Serial.print(dataArray[x]);
          //  Serial.print(x);
        }
        dataFile.println();       // 2 byte am ende für \n
        dataFile.close();
        // RFID Cip beschreiben
        tft.setCursor(0, 20);
        tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
        tft.setTextSize(1);
        tft.print("RFID Chip neu erstellen...");
        while((rfid(0)) != 0){
          //      Serial.println(  rfid(0));  // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und SD Karte schreiben
   //       Serial.println("jetzt neuen chip anlegen");
          //rfid(0);
        }
        digitalWrite(piezoBuzzer, HIGH);               //set digital pin for piezoBuzzer
        delay(10);
        digitalWrite(piezoBuzzer, LOW);               //set digital pin for piezoBuzzer
        // Anzeige auf TFT Aukku Details
        // TFT neuer Akku wurde hinzugefügt 
        tft.setCursor(0, 30);
        tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
        tft.println("                          ");
        tft.setCursor(0, 30);
        tft.print("Neuen Akku mit der ID: ");
        tft.println(akkuid);
//        tft.setCursor(80, 40);
        if(ndef){
        tft.print("NDEF Tag ");
        }
        else{
        tft.print("Empty Tag ");
        }          
        tft.println("angelegt. ");

        //tft.println("wurde erfolgreich angelegt");
        delay(5000);
        tft.fillScreen(ST7735_BLACK);          // init complete screen with black color
        akkuDetailsAnzeigen();

        // print to the serial port too: Debug
        // Serial.println(dataArray);
      }  
      // if the file isn't open, pop up an error:
      else{
        //     tft.drawString(0, 5, "error opening akkus.csv for write", WHITE);       
        tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
        tft.setCursor(10, 20);

        tft.println("error opening akkus.csv");
        tft.println("for write new akku from serial");
        while(1);
        return;
      } 

    }
    if (befehl == "r"){
      // open sd card to read all Data to Serial
      //Serial.println();
      dataFile = SD.open("akkus.csv", FILE_READ);
      // if the file opened okay, write to it:

      if (dataFile){
        tft.fillScreen(ST7735_BLACK);                                                  
        tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
        tft.setTextSize(2);
        tft.setCursor(0, 0);
        tft.println("   PC Modus  ");
        tft.setTextSize(1);
        tft.setCursor(0, 30);   
        tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
        tft.println("Reading akkus.csv");
        tft.println("send data to Serial....");  
        if (! RTC.isrunning()) {
              Serial.println("RTC is NOT running!");
        }
        else{
            Serial.print("RTC DateTime: ");
            Serial.print(now.day(), DEC);
            Serial.print('/');
            Serial.print(now.month(), DEC);
            Serial.print('/');
            Serial.print(now.year(), DEC);
            Serial.print(' ');
            Serial.print(now.hour(), DEC);
            Serial.print(':');
            Serial.print(now.minute(), DEC);
            Serial.print(':');
            Serial.print(now.second(), DEC);
            Serial.println();
        }
         
        int value = 0;
        // read from the file until there's nothing else in it:
        while (dataFile.available()){
          Serial.write(dataFile.read());
          value ++;
        }
        tft.setCursor(0, 60);   
        tft.setTextColor(ST7735_CYAN,ST7735_BLACK);
        tft.print(value);
        tft.println(" Byte entspricht ");
        tft.print(value/114);
        tft.println(" Datensaetzen wurden");
        tft.println("erfolgreich gesendet");
        // close the file: to write the Data flush also does it without close
        dataFile.close();

      }

      else {
        // if the file didn't open, print an error:
        tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
        tft.setCursor(0, 30);
        tft.println("error open akkus.csv ");
        tft.println("read sd card to serial");
        //   Serial.println("error opening akkus.csv for read");
        while(1);
        // delay(100);
        //  return;
      }
    }

    // clear the string:
    inputString = "";
    Serial.flush();
    stringComplete = false;
  }  // End of stringComplete

}	//End of serialData

/*
*
 splitString need the String to split, the parser ',' or ';' for example
 and the index of the Position in String after parser
 returns Substring
 String marker=splitString(test,',',0);
 Serial.println(marker)
 *
 */
String splitString(String s, char parser,int index){
  String rs="\0"; //String rs='\0';
  int parserIndex = index;
  int parserCnt=0;
  int rFromIndex=0, rToIndex=-1;

  while(index>=parserCnt){
    rFromIndex = rToIndex+1;
    rToIndex = s.indexOf(parser,rFromIndex);
    if(index == parserCnt){
      if(rToIndex == 0 || rToIndex == -1){
        return "\0";  // return '\0';
      }
      return s.substring(rFromIndex,rToIndex);
    }
    else{
      parserCnt++;
    }
  }
  return rs;
}

/*
  SerialEvent occurs whenever a new data comes in the
 hardware serial RX.  This routine is run between each
 time loop() runs, so using delay inside loop can delay
 response.  Multiple bytes of data may be available.
 */
void serialEvent(){
  while (Serial.available()){
    // get the new byte:
    char inChar = (char)Serial.read(); 
    // add it to the inputString:
    inputString += inChar;
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\n'){
      stringComplete = true;
    } 
  }
}

void akkuDetailsAnzeigen(){     //Standard Anzeige für Akkudaten 
  
  tft.setCursor(0, 30);
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.println("                          ");
  tft.setCursor(40, 30);
  tft.setTextColor(ST7735_CYAN,ST7735_BLACK);
  tft.println(akkuname);
  tft.setCursor(0, 40);
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.print("Typ:     ");
  tft.setTextColor(ST7735_YELLOW,ST7735_BLACK);
  tft.print(akkutyp);
  tft.setTextColor(ST7735_YELLOW,ST7735_BLACK);
  tft.print(zellenzahl);
  tft.setTextColor(ST7735_CYAN,ST7735_BLACK);
  tft.print("S ");
  tft.setTextColor(ST7735_YELLOW,ST7735_BLACK);
  tft.println(akkuah + "." + akkumah + "Ah");
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.print("Ladestrom:    ");
  tft.setTextColor(ST7735_GREEN,ST7735_BLACK);
  tft.println(maxladestromA + "." + maxladestrommA + "A");
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.print("Entladestrom: ");
  tft.setTextColor(ST7735_CYAN,ST7735_BLACK);
  tft.println(maxentladestromA + "." + maxentladestrommA + "A");
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.print("Kaufdatum:    ");
  tft.setTextColor(ST7735_YELLOW,ST7735_BLACK);
  tft.println(akkukauftag + "." + akkukaufmonat + "." + "20" + akkukaufjahr);
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.print("Scan:"); 
  tft.setTextColor(ST7735_CYAN,ST7735_BLACK);
  tft.print(akkuscanstunde + ":" + akkuscanminute + ":" + akkuscansekunde + " ");
  tft.println(akkuscantag + "." + akkuscanmonat + "." + "20" + akkuscanjahr);
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.print("Kaufpreis:    ");
  tft.setTextColor(ST7735_YELLOW,ST7735_BLACK);
  tft.println(preiseuro + "." + preiseurocent + " Euro");
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.print("AkkuID:       ");
  tft.setTextColor(ST7735_CYAN,ST7735_BLACK);
  tft.println(akkuid);
  if (ndef){
      tft.setTextColor(ST7735_YELLOW,ST7735_BLACK);
      tft.print("NDEF Tag");
  }
  else{
      tft.setTextColor(ST7735_YELLOW,ST7735_BLACK);
      tft.print("Empty Tag");
  }
  tft.setCursor(0, 105);
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);
  tft.setTextSize(2);
  tft.print("Loads: ");
  tft.setCursor(75, 100);
  tft.setTextColor(ST7735_RED,ST7735_BLACK);
  tft.setTextSize(3);
  tft.println(ladungen);
  tft.setTextSize(1);

}

int rfid(int modus){
  // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und array6 und SD Karte schreiben
  //*************************************************************************************
  // ab hier RFID
  //  noInterrupts();
//  Serial.print("Tag Leseversuch :");
//  Serial.println(modus);

  int val=50;
  uchar i,tmp;
  uchar status;
  uchar str[MAX_LEN];
  uchar RC_size;
  uchar blockAddr;	//A selection operation block addresses 0 to 63
  ndef=false;
  // das ist nur für rfid
  //Find the card, return the card type
 if (selectCard(true))
    {
    val=21;   // Card Erkennung war OK
//    Serial.println("Tag wurde erkannt");

  if (modus == 0){      
    //Registration card
    blockAddr = 15;		//Data block 11		
    status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
                   val=52;
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
                   val=53;
              }
    if (status == MI_OK){
        if (val==52 || val==53){
            ndef=true;
//            Serial.println("Es ist ein NDEF Tag");
            val=21;
        }
      blockAddr = 14 ; 
      status = MFRC522_Write(blockAddr, writeDataAkkuName);
      if(status == MI_OK){
//        Serial.println("The card has set the block 14 to writeDataAkkuName data !");
        val++; // Alles biser ok sogar das schreiben
      }
    }
    blockAddr = 15;
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){
      blockAddr = 13 ; 
      status = MFRC522_Write(blockAddr, writeDataAkkuDetails1);
      if(status == MI_OK){
//        Serial.println("The card has set the block 13 to writeDataAkkuDetails1 data !");
        val++; // Alles biser ok sogar das schreiben
      }
    }
    blockAddr = 15;
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){
      blockAddr = 12 ;
      status = MFRC522_Write(blockAddr, writeDataAkkuDetails2);
      if(status == MI_OK){
//        Serial.println("The card has set the block 12 to writeDataAkkuDetails2 data !");
        val++; // Alles biser ok sogar das schreiben
      }
    }

    blockAddr = 11;
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){  
      blockAddr = 10 ; 
      status = MFRC522_Write(blockAddr, writeDataAkkuDetails3);
      if(status == MI_OK){
//        Serial.println("The card has set the block 10 to writeDataAkkuDetails3 data !");
        val++; // Alles biser ok sogar das schreiben
      }  
    } 
    blockAddr = 11;
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){  
      blockAddr = 9 ; 
      status = MFRC522_Write(blockAddr, writeDataAkkuDetails4);
      if(status == MI_OK){
//        Serial.println("The card has set the block 9 to writeDataAkkuDetails4 data !");
        val++; // Alles biser ok sogar das schreiben
      }
    }
    blockAddr = 11;
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){   
      blockAddr = 8 ; 
      status = MFRC522_Write(blockAddr, writeDataAkkuDetails5);
      if(status == MI_OK){
//        Serial.println("The card has set the block 8 to writeDataAkkuDetails5 data !");
        val++; // Alles biser ok sogar das schreiben
      }  
    }
    blockAddr =7;
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){
      blockAddr = 6; 
      status = MFRC522_Write(blockAddr, writeDataAkkuDetails6);
      if(status == MI_OK){
//        Serial.println("The card has set the block 6 to writeDataAkkuDetails6 data !");
        val++; // Alles biser ok sogar das schreiben 28 wenn alles ok
//        Serial.println(val);
      } 
    }
  }

 if ((modus ==1) or (modus ==2)){
    //Card Reader
    blockAddr = 15;		//Data block 11		
    //		status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
                   val=54;
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
                   val=55;
              }

    if (status == MI_OK){
       if (val==54 || val==55){
             ndef=true;
//             Serial.println("Es ist ein NDEF Tag");
       }
      //Read data

      blockAddr = 14 ; 
      status = MFRC522_Read(blockAddr, str);
      if (status == MI_OK){
//        Serial.println("Read from the card block 14 ,the data is : ");
        for  (i=0; i<16; i++){
          writeDataAkkuName[i] = str[i];
//          Serial.print(str[i],DEC);
//          Serial.print(" , ");
         }
//        Serial.println(" ");
//        //Read data
        val=30;
      }
    }	
    blockAddr = 15;		//Data block 11		
    //		status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){
      blockAddr =  13 ; 
      status = MFRC522_Read(blockAddr, str);
      if (status == MI_OK){
//        Serial.println("Read from the card block 13 ,the data is : ");
        for (i=0; i<16; i++){
          writeDataAkkuDetails1[i] = str[i];
//          Serial.print(str[i],DEC);
//          Serial.print(" , ");
        }
//        Serial.println(" ");
        val++;
      }
      //Read data
    }
    blockAddr = 15;		//Data block 11		
    //		status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){

      blockAddr = 12 ; 
      status = MFRC522_Read(blockAddr, str);
      if (status == MI_OK){
//        Serial.println("Read from the card block 12 ,the data is : ");
        for (i=0; i<16; i++){
          writeDataAkkuDetails2[i] = str[i];
//          Serial.print(str[i],DEC);
//          Serial.print(" , ");
        }
//        Serial.println(" ");
//        //Read data
        val++;
      } 
    }

    blockAddr = 11;		//Data block 11		
    //		status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){

      blockAddr = 10; 
      status = MFRC522_Read(blockAddr, str);
      if (status == MI_OK){
//        Serial.println("Read from the card block 10 ,the data is : ");
        for (i=0; i<16; i++){
          writeDataAkkuDetails3[i] = str[i];
//          Serial.print(str[i],DEC);
//          Serial.print(" , ");
        }
//        Serial.println(" ");
        //Read data
        val++;
      }
    }
    blockAddr = 11;		//Data block 11		
    //		status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){

      blockAddr = 9 ; 
      status = MFRC522_Read(blockAddr, str);
      if (status == MI_OK){
//        Serial.println("Read from the card block 9 ,the data is : ");
        for (i=0; i<16; i++){
          writeDataAkkuDetails4[i] = str[i];
//          Serial.print(str[i],DEC);
//          Serial.print(" , ");
        }
//        Serial.println(" ");
        //Read data
        val++;
      }
    }
    blockAddr = 11;		//Data block 11		
    //		status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){

      blockAddr = 8; 
      status = MFRC522_Read(blockAddr, str);
      if (status == MI_OK){
//        Serial.println("Read from the card block 8 ,the data is : ");
        for (i=0; i<16; i++){
          writeDataAkkuDetails5[i] = str[i];
//          Serial.print(str[i],DEC);
//          Serial.print(" , ");
        }
//        Serial.println(" ");
        //Read data
        val++;
      }
    }
    blockAddr = 7;		//Data block 11		
    //		status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){

      blockAddr = 6 ; 
      status = MFRC522_Read(blockAddr, str);
      if (status == MI_OK){
//        Serial.println("Read from the card block 6 ,the data is : ");
        for (i=0; i<16; i++){
          writeDataAkkuDetails6[i] = str[i];
//          Serial.print(str[i],DEC);
//          Serial.print(" , ");
        }
        val++;    //Wenn alles ok dann ist hier 36
      }
//      Serial.println();
    }
  }
//  Serial.println();
//  Serial.println(val);
   if (val==36){ //lesen alles OK
  //   Die rfid daten sind im array als byte vorhanden und müssen nach Char konvertiert werden.
  //   Zeige die Daten auf dem TFT Display an 
  //   wenn eine 2 übergeben wird, dann erhöhe die akkuid um eins und schreibe inkl. aktuelle Scan Datum ins ARRAY3 
  // teststring    n;Lipo ;TurnigyNanotec;12;3;850;10;50;235;70;31;12;13;10;
    
  // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und SD Karte schreiben
  //Put now all data to the tft Display
  // Serial.println("wait rfdi !=1");
  // rfid(1);  // 0 = RFID neuen Chip anlegen, 1 = vorhandenden Chip nur lesen, 2 = vorhandenen Chip lesen und die neue Daten in array 3 und SD Karte schreiben
 

  //     Aukkuname Array auslesen
  char array[16];
  for (int x=0; x <= 15  ; x++){
    char mychar=writeDataAkkuName[x];  
    array[x] = mychar;
  }
  //     Aukkudaten Array1 auslesen
  char array1[16];
  for (int x=0; x <= 15  ; x++){
    char mychar=writeDataAkkuDetails1[x];  
    array1[x] = mychar;
  }
  //     Aukkudaten Array2 auslesen
  char array2[16];
  for (int x=0; x <= 15  ; x++){
    char mychar=writeDataAkkuDetails2[x];  
    array2[x] = mychar;
  }
  //     Aukkudaten Array3 auslesen
  char array3[16];
  for (int x=0; x <= 15  ; x++){
    char mychar=writeDataAkkuDetails3[x];  
    array3[x] = mychar;
  }
  //     Aukkudaten Array4 auslesen
  char array4[16];
  for (int x=0; x <= 15  ; x++){
    char mychar=writeDataAkkuDetails4[x];  
    array4[x] = mychar;
  }
  //     Aukkudaten Array5 auslesen
  char array5[16];
  for (int x=0; x <= 15  ; x++){
    char mychar=writeDataAkkuDetails5[x];  
    array5[x] = mychar;
  }//     Aukkudaten Array6 auslesen
  char array6[16];
  for (int x=0; x <= 15  ; x++){
    char mychar=writeDataAkkuDetails6[x];  
    array6[x] = mychar;
  }
//  char array7[5];
//  for (int x=0; x < 4  ; x++){
//    char mychar=serNum[x];  
//    array7[x] = mychar;
//  }
  zellenzahl=splitString(array1,';',0);
  akkuah=splitString(array1,';',1);
  akkumah=splitString(array1,';',2);
  akkukauftag=splitString(array2,';',0);
  akkukaufmonat=splitString(array2,';',1);
  akkukaufjahr=splitString(array2,';',2);
  akkutyp=splitString(array2,';',3);
  akkuscantag=splitString(array3,';',0);
  akkuscanmonat=splitString(array3,';',1);
  akkuscanjahr=splitString(array3,';',2);
  akkuid=splitString(array3,';',3);
  ladungen=splitString(array3,';',4);
  maxentladestromA=splitString(array4,';',0);
  maxentladestrommA=splitString(array4,';',1);
  maxladestromA=splitString(array4,';',2);
  maxladestrommA=splitString(array4,';',3);
  akkuname=splitString(array,';',0);
  preiseuro=splitString(array5,';',0);
  preiseurocent=splitString(array5,';',1);
  maxgeladenemah=splitString(array5,';',2);
  maxgeladenema=splitString(array5,';',3);
  akkuscanstunde=splitString(array6,';',0);
  akkuscanminute=splitString(array6,';',1);
  akkuscansekunde=splitString(array6,';',2);
  lastgeladenema=splitString(array6,';',3);
  
     val=1;
  }

  
  if ((modus == 2) and (val ==1)){
    //Recharge	
    DateTime now = RTC.now();
    val=40;
    akkuscantag = String(now.day());  // aus RTC
    akkuscanmonat = String(now.month()); // aus RTC
    int    ajahr = now.year(); // aus RTC
    akkuscanstunde = String(now.hour());  // aus RTC
    akkuscanminute = String(now.minute());  // aus RTC
    akkuscansekunde = String(now.second());  // aus RTC
    int value = ladungen.toInt();
    lastgeladenema="0";
    ajahr = ajahr -2000;
    akkuscanjahr = String(ajahr);
    value++;
    ladungen = String(value);
    String val20 = String(akkuscantag + ";" + akkuscanmonat + ";" + ajahr + ";" + akkuid  + ";" + ladungen + ";"); // 16 Bytearray
    //        Serial.println(val3.length());
   // Serial.println(val20);
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails3[x] = val20[x];
  //     Serial.print(writeDataAkkuDetails3[x]);
      }
    blockAddr = 11;		//Data block 11		
    //status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){
      blockAddr = 10 ;
      status = MFRC522_Write(blockAddr, writeDataAkkuDetails3);
      if (status == MI_OK){
        //write data
         val++;
      }
    }
     String val21 = String(akkuscanstunde + ";" + akkuscanminute + ";" + akkuscansekunde + ";" +  lastgeladenema + ";"); // 16 Bytearray
    // Serial.println(val21);
     //       Serial.println(val3.length());
      for (int x=0; x <= 15  ; x++){
        writeDataAkkuDetails6[x] = val21[x];
  //     Serial.print(writeDataAkkuDetails3[x]);
      }

    blockAddr = 7;		//Data block 11		
    //status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr,, sectorNewKeyA[blockAddr/4], serNum);	//Authenticate
        status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, defaultKeyA, serNum); //auth with default key
              if (status != MI_OK)
              {
                   selectCard(false);
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, madKeyA, serNum); //auth with MAD key
              }
              if (status != MI_OK)
              {
                   selectCard(false);   
                   status = MFRC522_Auth(PICC_AUTHENT1A, blockAddr, NDEFKeyA, serNum); //auth NDEF data key
              }

    if (status == MI_OK){
      blockAddr = 6;
      status = MFRC522_Write(blockAddr, writeDataAkkuDetails6);
      if (status == MI_OK){
        //write data
       
        if (val==41){
          val=2;
        }
       }
      
    }
 //   Serial.println(" ");
  }
  if (val==28){ // Modus 0 neue karte alles ok
//    Serial.println("Schreiben OK");
    val=0;
  }
  // Serial.println(val,DEC);
} // vom select card
  MFRC522_Halt();			//Command card into hibernation    
  delay(50);  
//  Serial.println(val);
  return val;
}
//******************************************************************************   RFID ENDE



/*
 * Function name: Write_MFRC5200
 * Description: write a byte of data to a register in MFRC522
 * Input parameters: addr - the register address; val - value to be written
 * Return value: no
 */
void Write_MFRC522(uchar addr, uchar val)
{
  digitalWrite(chipSelectPin, LOW);

  //Address Format：0XXXXXX0
  SPI.transfer((addr<<1)&0x7E);	
  SPI.transfer(val);

  digitalWrite(chipSelectPin, HIGH);
}


/*
 * Function name: Read_MFRC522
 * Description: read a byte of data from a register of MFRC522
 * Input parameters: addr - register address
 * Return value: Returns a byte read data
 */
uchar Read_MFRC522(uchar addr)
{
  uchar val;

  digitalWrite(chipSelectPin, LOW);

  //Address Format：1XXXXXX0
  SPI.transfer(((addr<<1)&0x7E) | 0x80);	
  val =SPI.transfer(0x00);

  digitalWrite(chipSelectPin, HIGH);

  return val;	
}

uchar MFRC522_SelectTag2(uchar *serNum, uchar *sak)
{
    uchar i;
    uchar status;
    //uchar size;
    uint recvBits;
    uchar buffer[9];
    //uchar buffCheck=0; 

    //ClearBitMask(Status2Reg, 0x08);			//MFCrypto1On=0

    buffer[0] = PICC_ANTICOLL2;
    buffer[1] = 0x70;
    for (i=0; i<5; i++)
    {
    	buffer[i+2] = *(serNum+i);
    }
    CalulateCRC(buffer, 7, &buffer[7]);
    status = MFRC522_ToCard(PCD_TRANSCEIVE, buffer, 9, buffer, &recvBits);
    //TODO: the above call returns 2 instead of MI_OK -> why?
    status = MI_OK;
    //Serial.print("recvBits: ");
    //Serial.print(recvBits, DEC);
    /*
    for (i=0; i<recBits / 8; i++)
    {
    	buff[i] = *(buffer+i);
    }*/
    //dumpHex((char*)buffer, recvBits / 8);
    *sak = buffer[0];
    //Verify received buffer
    /* TODO
    for (i=0; i< recvBits/8; i++)
    {   
       buffCheck ^= buffer[i];
    }
    if (buffCheck != buffer[i])
    {   
       status = MI_ERR;    
    }*/
    return status;
}


/*
 * Function name: SetBitMask
 * Description: set RC522 register bit
 * Input: reg - register address; mask - set value
 * Return value: no
 */
void SetBitMask(uchar reg, uchar mask)  
{
  uchar tmp;
  tmp = Read_MFRC522(reg);
  Write_MFRC522(reg, tmp | mask);  // set bit mask
}


/*
 * Function name: ClearBitMask
 * Description: clear RC522 register bit
 * Input: reg - register address; mask - Qing-bit value
 * Return value: no
 */
void ClearBitMask(uchar reg, uchar mask)  
{
  uchar tmp;
  tmp = Read_MFRC522(reg);
  Write_MFRC522(reg, tmp & (~mask));  // clear bit mask
} 


/*
 * Function name: AntennaOn
 * Description: Open antennas, each time you start or close a natural barrier between the emission should be at least 1ms intervals
 * Input: None
 * Return value: no
 */
void AntennaOn(void)
{
  uchar temp;

  temp = Read_MFRC522(TxControlReg);
  if (!(temp & 0x03))
  {
    SetBitMask(TxControlReg, 0x03);
  }
}


/*
 * Function name: AntennaOff
 * Description: Close antennas, each time you start or close a natural barrier between the emission should be at least 1ms intervals
 * Input: None
 * Return value: no
 */
void AntennaOff(void)
{
  ClearBitMask(TxControlReg, 0x03);
}


/*
 * Function name: ResetMFRC522
 * Description: Reset RC522
 * Input: None
 * Return value: no
 */
void MFRC522_Reset(void)
{
  Write_MFRC522(CommandReg, PCD_RESETPHASE);
}


/*
 * Function name: InitMFRC522
 * Description: Initialize RC522
 * Input: None
 * Return value: no
 */
void MFRC522_Init(void)
{
  digitalWrite(NRSTPD,HIGH);

  MFRC522_Reset();

  //Timer: TPrescaler*TreloadVal/6.78MHz = 24ms
  Write_MFRC522(TModeReg, 0x8D);		//Tauto=1; f(Timer) = 6.78MHz/TPreScaler
  Write_MFRC522(TPrescalerReg, 0x3E);	//TModeReg[3..0] + TPrescalerReg
  Write_MFRC522(TReloadRegL, 30);           
  Write_MFRC522(TReloadRegH, 0);

  Write_MFRC522(TxAutoReg, 0x40);		//100%ASK
  Write_MFRC522(ModeReg, 0x3D);		//CRC initial value0x6363	???

  //ClearBitMask(Status2Reg, 0x08);		//MFCrypto1On=0
  //Write_MFRC522(RxSelReg, 0x86);		//RxWait = RxSelReg[5..0]
  //Write_MFRC522(RFCfgReg, 0x7F);   		//RxGain = 48dB

  AntennaOn();		//Open the antenna
}


/*
 * Function: MFRC522_Request
 * Description: Find cards, read the card type No.
 * Input parameters: reqMode - find cards way
 * TagType - Back to the card type 
 *			 	0x4400 = Mifare_UltraLight
 *				0x0400 = Mifare_One(S50)
 *				0x0200 = Mifare_One(S70)
 *				0x0800 = Mifare_Pro(X)
 *				0x4403 = Mifare_DESFire
 * Return value: the successful return MI_OK
 */
uchar MFRC522_Request(uchar reqMode, uchar *TagType)
{
  uchar status;  
  uint backBits;			//The received data bits

    Write_MFRC522(BitFramingReg, 0x07);		//TxLastBists = BitFramingReg[2..0]	???

  TagType[0] = reqMode;
  status = MFRC522_ToCard(PCD_TRANSCEIVE, TagType, 1, TagType, &backBits);

  if ((status != MI_OK) || (backBits != 0x10))
  {    
    status = MI_ERR;
  }

  return status;
}


/*
 * Function name: MFRC522_ToCard
 * Description: RC522 and ISO14443 card communication
 * Input Parameters: command - MF522 command word,
 * SendData - RC522 sent to the card data,
 * SendLen - length of data sent
 * BackData - received the card data is returned,
 * BackLen - returns the bit length of the data
 * Return value: the successful return MI_OK
 */
uchar MFRC522_ToCard(uchar command, uchar *sendData, uchar sendLen, uchar *backData, uint *backLen)
{
  uchar status = MI_ERR;
  uchar irqEn = 0x00;
  uchar waitIRq = 0x00;
  uchar lastBits;
  uchar n;
  uint i;

  switch (command)
  {
  case PCD_AUTHENT:		//Certification cards close
    {
      irqEn = 0x12;
      waitIRq = 0x10;
      break;
    }
  case PCD_TRANSCEIVE:	//Transmit FIFO data
    {
      irqEn = 0x77;
      waitIRq = 0x30;
      break;
    }
  default:
    break;
  }

  Write_MFRC522(CommIEnReg, irqEn|0x80);	//Interrupt request
  ClearBitMask(CommIrqReg, 0x80);			//Clear all interrupt request bit
  SetBitMask(FIFOLevelReg, 0x80);			//FlushBuffer=1, FIFO initialization

  Write_MFRC522(CommandReg, PCD_IDLE);	//NO action;Cancel the current command	???

  //Data is written to the FIFO
  for (i=0; i<sendLen; i++)
  {   
    Write_MFRC522(FIFODataReg, sendData[i]);    
  }

  //Execute commands
  Write_MFRC522(CommandReg, command);
  if (command == PCD_TRANSCEIVE)
  {    
    SetBitMask(BitFramingReg, 0x80);		//StartSend=1,transmission of data starts  
  }   

  //Waiting for the completion of the reception data
  i = 2000;	//i according to the clock frequency adjustment, the maximum waiting time of the operation M1 card 25ms	???
  do 
  {
    //CommIrqReg[7..0]
    //Set1 TxIRq RxIRq IdleIRq HiAlerIRq LoAlertIRq ErrIRq TimerIRq
    n = Read_MFRC522(CommIrqReg);
    i--;
  }
  while ((i!=0) && !(n&0x01) && !(n&waitIRq));

  ClearBitMask(BitFramingReg, 0x80);			//StartSend=0

    if (i != 0)
  {    
    if(!(Read_MFRC522(ErrorReg) & 0x1B))	//BufferOvfl Collerr CRCErr ProtecolErr
    {
      status = MI_OK;
      if (n & irqEn & 0x01)
      {   
        status = MI_NOTAGERR;			//??   
      }

      if (command == PCD_TRANSCEIVE)
      {
        n = Read_MFRC522(FIFOLevelReg);
        lastBits = Read_MFRC522(ControlReg) & 0x07;
        if (lastBits)
        {   
          *backLen = (n-1)*8 + lastBits;   
        }
        else
        {   
          *backLen = n*8;   
        }

        if (n == 0)
        {   
          n = 1;    
        }
        if (n > MAX_LEN)
        {   
          n = MAX_LEN;   
        }

        //The received data in the read FIFO
        for (i=0; i<n; i++)
        {   
          backData[i] = Read_MFRC522(FIFODataReg);    
        }
      }
    }
    else
    {   
      status = MI_ERR;  
    }

  }

  //SetBitMask(ControlReg,0x80);           //timer stops
  //Write_MFRC522(CommandReg, PCD_IDLE); 

  return status;
}


/*
 * Function name: MFRC522_Anticoll
 * Description: Anti-collision detection, read the card serial number of the selected card
 * Input parameters: serNum - returns the 4 byte card serial number, the 5-byte checksum byte
 * Return value: the successful return MI_OK
 */
uchar MFRC522_Anticoll(uchar *serNum)
{
  uchar status;
  uchar i;
  uchar serNumCheck=0;
  uint unLen;


  //ClearBitMask(Status2Reg, 0x08);		//TempSensclear
  //ClearBitMask(CollReg,0x80);			//ValuesAfterColl
  Write_MFRC522(BitFramingReg, 0x00);		//TxLastBists = BitFramingReg[2..0]

  serNum[0] = PICC_ANTICOLL;
  serNum[1] = 0x20;
  status = MFRC522_ToCard(PCD_TRANSCEIVE, serNum, 2, serNum, &unLen);

  if (status == MI_OK)
  {
    //Check card serial number
    for (i=0; i<4; i++)
    {   
      serNumCheck ^= serNum[i];
    }
    if (serNumCheck != serNum[i])
    {   
      status = MI_ERR;    
    }
  }

  //SetBitMask(CollReg, 0x80);		//ValuesAfterColl=1

  return status;
} 

//ANTICOLL cascading level 2
uchar MFRC522_Anticoll2(uchar *serNum)
{
    uchar status;
    uchar i;
    uchar serNumCheck=0;
    uint unLen;
    

    //ClearBitMask(Status2Reg, 0x08);		//TempSensclear
    //ClearBitMask(CollReg,0x80);		//ValuesAfterColl
    Write_MFRC522(BitFramingReg, 0x00);		//TxLastBists = BitFramingReg[2..0]
 
    serNum[0] = PICC_ANTICOLL2;
    serNum[1] = 0x20;
    status = MFRC522_ToCard(PCD_TRANSCEIVE, serNum, 2, serNum, &unLen);

    if (status == MI_OK)
    {
	 //Verify card serial number
         for (i=0; i<4; i++)
	 {    
	     serNumCheck ^= serNum[i];
	 }
	 if (serNumCheck != serNum[i])
	 {   
	     status = MI_ERR;    
	 }
    }
    //SetBitMask(CollReg, 0x80);		//ValuesAfterColl=1
    return status;
}

//send RATS and returns ATS
uchar MFRC522_RATS(uchar *recvData, uint *pLen)
{
    uchar status;
    uint unLen = 0;

    recvData[0] = 0xE0; // RATS
    recvData[1] = 0x50; // FSD=128, CID=0
    CalulateCRC(recvData,2, &recvData[2]);
    status = MFRC522_ToCard(PCD_TRANSCEIVE, recvData, 4, recvData, &unLen);
    /*
    Serial.print("unLen: ");
    Serial.print(unLen, HEX);
    /*Serial.print(" status: ");
    Serial.print(status, HEX);
    Serial.println("");
    */
    //TODO
    //if ((status != MI_OK) || (unLen != 0x90))
    //{
    //    status = MI_ERR;
    //}
    return status;
}

/*
 * Function name: CalulateCRC
 * Description: MF522 calculated CRC
 * Input parameters: pIndata - CRC data, len - the length of the data, pOutData - calculate CRC result to be reading
 * Return value: no
 */
void CalulateCRC(uchar *pIndata, uchar len, uchar *pOutData)
{
  uchar i, n;

  ClearBitMask(DivIrqReg, 0x04);			//CRCIrq = 0
  SetBitMask(FIFOLevelReg, 0x80);			//Clear FIFO pointer
  //Write_MFRC522(CommandReg, PCD_IDLE);

  //Data is written to the FIFO
  for (i=0; i<len; i++)
  {   
    Write_MFRC522(FIFODataReg, *(pIndata+i));   
  }
  Write_MFRC522(CommandReg, PCD_CALCCRC);

  //Wait for the CRC calculation is complete
  i = 0xFF;
  do 
  {
    n = Read_MFRC522(DivIrqReg);
    i--;
  }
  while ((i!=0) && !(n&0x04));			//CRCIrq = 1

  //To read CRC calculation results
  pOutData[0] = Read_MFRC522(CRCResultRegL);
  pOutData[1] = Read_MFRC522(CRCResultRegM);
}


/*
 * Function: MFRC522_SelectTag
 * Description: Select card, read the card memory capacity
 * Input parameters: serNum - incoming card serial number
 * Return value: the successful return of the card capacity
 */
//uchar MFRC522_SelectTag(uchar *serNum)
//{
//  uchar i;
//  uchar status;
//  uchar size;
//  uint recvBits;
//  uchar buffer[9]; 
//
//  //ClearBitMask(Status2Reg, 0x08);			//MFCrypto1On=0
//
//  buffer[0] = PICC_SElECTTAG;
//  buffer[1] = 0x70;
//  for (i=0; i<5; i++)
//  {
//    buffer[i+2] = *(serNum+i);
//  }
//  CalulateCRC(buffer, 7, &buffer[7]);		//??
//  status = MFRC522_ToCard(PCD_TRANSCEIVE, buffer, 9, buffer, &recvBits);
//
//  if ((status == MI_OK) && (recvBits == 0x18))
//  {   
//    size = buffer[0]; 
//  }
//  else
//  {   
//    size = 0;    
//  }
//
//  return size;
//}

/*
 * Function:MFRC522_SelectTag
 * Description:Select card, read card storage volume
 * Input parameter:serNum--Send card serial number
 * sak see ISO14443-3 Table 9 — Coding of SAK
 * return return MI_OK if successed
 */
uchar MFRC522_SelectTag(uchar *serNum, uchar *sak)
{
    uchar i;
    uchar status;
    //uchar size;
    uint recvBits;
    uchar buffer[9];
    //uchar buffCheck=0; 

    //ClearBitMask(Status2Reg, 0x08);			//MFCrypto1On=0

    buffer[0] = PICC_SElECTTAG;
    buffer[1] = 0x70;
    for (i=0; i<5; i++)
    {
    	buffer[i+2] = *(serNum+i);
    }
    CalulateCRC(buffer, 7, &buffer[7]);		//??
    status = MFRC522_ToCard(PCD_TRANSCEIVE, buffer, 9, buffer, &recvBits);
    //TODO: the above call returns 2 instead of MI_OK -> why?
    status = MI_OK;
    //Serial.print("recvBits: ");
    //Serial.print(recvBits, DEC);
    /*
    for (i=0; i<recBits / 8; i++)
    {
    	buff[i] = *(buffer+i);
    }*/
    //dumpHex((char*)buffer, recvBits / 8);
    *sak = buffer[0];
    //Verify received buffer
    /* TODO
    for (i=0; i< recvBits/8; i++)
    {   
       buffCheck ^= buffer[i];
    }
    if (buffCheck != buffer[i])
    {   
       status = MI_ERR;    
    }*/
    return status;
}


uchar MFRC522_Auth(uchar authMode, uchar BlockAddr, uchar *Sectorkey, uchar *serNum)
{
  uchar status;
  uint recvBits;
  uchar i;
  uchar buff[12]; 

  //Verify instructions + block address + sectors password + card serial number
  buff[0] = authMode;
  buff[1] = BlockAddr;
  for (i=0; i<6; i++)
  {    
    buff[i+2] = *(Sectorkey+i);   
  }
  for (i=0; i<4; i++)
  {    
    buff[i+8] = *(serNum+i);   
  }
  status = MFRC522_ToCard(PCD_AUTHENT, buff, 12, buff, &recvBits);

  if ((status != MI_OK) || (!(Read_MFRC522(Status2Reg) & 0x08)))
  {   
    status = MI_ERR;   
  }

  return status;
}


/*
 * Function name: MFRC522_Read
 * Description: Read block data
 * Input parameters: blockAddr - block address; recvData - read out blocks of data
 * Return value: the successful return MI_OK
 */
uchar MFRC522_Read(uchar blockAddr, uchar *recvData)
{
  uchar status;
  uint unLen;

  recvData[0] = PICC_READ;
  recvData[1] = blockAddr;
  CalulateCRC(recvData,2, &recvData[2]);
  status = MFRC522_ToCard(PCD_TRANSCEIVE, recvData, 4, recvData, &unLen);

  if ((status != MI_OK) || (unLen != 0x90))
  {
    status = MI_ERR;
  }

  return status;
}


/*
 * Function name: MFRC522_Write
 * Description: write block data
 * Input parameters: blockAddr - block address; writeData - 16 bytes of data to write to the block
 * Return value: the successful return MI_OK
 */
uchar MFRC522_Write(uchar blockAddr, uchar *writeData)
{
  uchar status;
  uint recvBits;
  uchar i;
  uchar buff[18]; 

  buff[0] = PICC_WRITE;
  buff[1] = blockAddr;
  CalulateCRC(buff, 2, &buff[2]);
  status = MFRC522_ToCard(PCD_TRANSCEIVE, buff, 4, buff, &recvBits);

  if ((status != MI_OK) || (recvBits != 4) || ((buff[0] & 0x0F) != 0x0A))
  {   
    status = MI_ERR;   
  }

  if (status == MI_OK)
  {
    for (i=0; i<16; i++)		//16Byte data written to the FIFO
    {    
      buff[i] = *(writeData+i);   
    }
    CalulateCRC(buff, 16, &buff[16]);
    status = MFRC522_ToCard(PCD_TRANSCEIVE, buff, 18, buff, &recvBits);

    if ((status != MI_OK) || (recvBits != 4) || ((buff[0] & 0x0F) != 0x0A))
    {   
      status = MI_ERR;   
    }
  }

  return status;
}


/*
 * Function: MFRC522_Halt
 * Description: command card into hibernation
 * Input: None
 * Return value: no
 */
void MFRC522_Halt(void)
{
  uchar status;
  uint unLen;
  uchar buff[4]; 

  buff[0] = PICC_HALT;
  buff[1] = 0;
  CalulateCRC(buff, 2, &buff[2]);

  status = MFRC522_ToCard(PCD_TRANSCEIVE, buff, 4, buff,&unLen);
}
//
//void int8_t satoi( char *str, int len ){  
//  int8_t r = 0;  
//  for(int i=0; i<len; i++){  
//   //Check if this is a number  
//   if ( str[i] < 0x3a && str[i] > 0x2f){  
//     // is a ASCII number, return it  
//     r = r * 10;  
//   r += (str[i]-0x30);  
//   }else{  
//    i = len; //exit!  
//    r = -1;  
//    break;  
//   }    
//  }  
//  return r;   
// }  
//
//void int8_t satoi( char *str ){  
//  int8_t r = 0;  
//  int len = strlen(str);  
//  for(int i=0; i<len; i++){  
//   //Check if this is a number  
//   if ( str[i] < 0x3a && str[i] > 0x2f){  
//     // is a ASCII number, return it  
//     r = r * 10;  
//   r += (str[i]-0x30);  
//   }else{  
//    i = len; //exit!  
//    r = -1;  
//    break;  
//   }    
//  }  
//  return r;   
// }  
//

boolean selectCard(boolean dumpInfo) 
{
  uchar status;
  uchar buffer[MAX_LEN];
  //Search card, return card types
  status = MFRC522_Request(PICC_REQIDL, buffer);//ShortFrame: 0x26 REQA (Request Type A)
  //status = MFRC522_Request(PICC_REQALL, buffer);//0x52 WUPA (Wake-Up)
  if (status == MI_OK)
  {
     if (dumpInfo)
     {
//         Serial.print("Card detected.\r\n ATQA:");
         dumpHex((char*)buffer, 2);
//         Serial.println(" ");
     }
     //Prevent conflict, return the 4 bytes Serial number of the card
     status = MFRC522_Anticoll(buffer);
     if (status == MI_OK)
     {
         memcpy(serNum, buffer, 5);
         uchar sak = 0;
         status = MFRC522_SelectTag(serNum, &sak);                   
         if (status == MI_OK && ((sak & 0x04) == 0x00))
         {        
             if (dumpInfo)
             {
//                 Serial.print(" UID: ");
                 dumpHex((char*)serNum, 4);
//                 Serial.println("");
             }
             if ((sak & 0x20) == 0x20)
             {
                 //ISO/IEC FCD 14443-3: Table 9 — Coding of SAK
                 //if (dumpInfo)
                 //    Serial.println(" UID complete, PICC compliant with ISO/IEC 14443-4");
                 //send RATS (Request for Answer To Select)
                 uchar ats[MAX_LEN];
                 uint unLen = 0;
                 status = MFRC522_RATS(ats, &unLen);
                 if (status == MI_OK && dumpInfo)
                 {
 //                     Serial.println(" ATS: ");
                      dumpHex((char*)ats, ats[0]);
 //                     Serial.println("");
                 }
             }
             if (dumpInfo)
             {
//                 Serial.print(" SAK: ");
//                 Serial.print(sak, HEX);
//                 Serial.println("");
             }     
             return true;
         }
         else
         {
             //cascading level 2
             memcpy(serNum7, &serNum[1], 3);//cascading L1
             status = MFRC522_Anticoll2(buffer);
             if (status == MI_OK)
             {
                 memcpy(&serNum7[3], buffer, 4);
                 status = MFRC522_SelectTag2(&serNum7[3], &sak);
                 if (dumpInfo)
                 {
//                    Serial.print(" UID: ");
                    dumpHex((char*)serNum7, 7);
//                    Serial.println("");
//                    Serial.print(" SAK: ");
//                    Serial.print(sak, HEX);
//                    Serial.println("");
                 }
                 return true;
             }
             else
             {
//                 Serial.println("ANTICOLL error: cascading level 2");
             }
         }
   }//Anticoll
   else
   {
//      Serial.print("ANTICOLL failed");
   }
 }
 else
 {
     //Serial.print("-");
 }
 return false;  
}//selectCard

void dumpHex(char* buffer, int len)
{
  for(byte i=0; i < len; i++) {
     char text[4];
     if (i % 16 == 0) {
//        Serial.print(" "); 
     }
     sprintf(text, "%02X \x00", (byte)(*(buffer + i)));
//     Serial.print(text);
 
     if (i % 16 == 15) {
//        Serial.println(""); 
     }
  }
  //Serial.println(" "); 
}

